# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

from onnx.helper import tensor_dtype_to_np_dtype
from onnx.onnx_pb import TensorProto
from onnx.reference.op_run import OpRun


class EyeLike(OpRun):
    def _run(self, data, *args, dtype=None, k=None):  # noqa: ARG002
        if dtype is None:
            if data is None:
                _dtype = np.float32
            else:
                _dtype = data.dtype
        elif dtype == TensorProto.STRING:
            _dtype = np.str_  # type: ignore[assignment]
        else:
            _dtype = tensor_dtype_to_np_dtype(dtype)  # type: ignore[assignment]
        shape = data.shape
        if len(shape) == 1:
            sh = (shape[0], shape[0])
        elif len(shape) == 2:
            sh = shape
        else:
            raise RuntimeError(f"EyeLike only accept 1D or 2D tensors not {shape!r}.")
        return (np.eye(*sh, k=k, dtype=_dtype),)



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

from onnx.reference.ops.aionnx_preview_training._op_run_training import OpRunTraining


def _apply_momentum(r, t, x, g, v, norm_coefficient, alpha, beta):
    # Add gradient of regularization term.
    g_regularized = norm_coefficient * x + g
    # Coefficient of gradient should be 1 at the first iteration.
    beta_adjusted = beta if t > 0 else 1
    # Update momentum.
    v_new = alpha * v + beta_adjusted * g_regularized
    # Apply SG with momentum update rule.
    x_new = x - r * v_new
    return x_new, v_new


def _apply_nesterov(r, t, x, g, v, norm_coefficient, alpha, beta):
    # Add gradient of regularization term.
    g_regularized = norm_coefficient * x + g
    # Coefficient of gradient should be 1 at the first iteration.
    beta_adjusted = beta if t > 0 else 1
    # Update momentum.
    v_new = alpha * v + beta_adjusted * g_regularized
    # Apply Nesterov with momentum update rule.
    x_new = x - r * (g_regularized + alpha * v_new)
    return x_new, v_new


class Momentum(OpRunTraining):
    def _run(self, *data, alpha=None, beta=None, mode=None, norm_coefficient=None):
        if len(data) == 5:
            r, t, x, g, v = data
            return self._run1(
                r,
                t,
                x,
                g,
                v,
                norm_coefficient=norm_coefficient,
                alpha=alpha,
                beta=beta,
                mode=mode,
            )
        n = (len(data) - 2) // 3
        xs = []
        vs = []
        for i in range(n):
            a, b = self._run1(
                *data[:2],
                data[2 + i],
                data[2 + n + i],
                data[2 + n * 2 + i],
                norm_coefficient=norm_coefficient,
                alpha=alpha,
                beta=beta,
                mode=mode,
            )
            xs.append(a)
            vs.append(b)
        return tuple(xs + vs)

    def _run1(
        self,
        r,
        t,
        x,
        g,
        v,
        mode="standard",
        norm_coefficient=None,
        alpha=None,
        beta=None,
    ):
        if mode == "standard":
            x_new, v_new = _apply_momentum(r, t, x, g, v, norm_coefficient, alpha, beta)
        else:
            x_new, v_new = _apply_nesterov(r, t, x, g, v, norm_coefficient, alpha, beta)
        return x_new, v_new



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Version(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    _START_VERSION: _ClassVar[Version]
    IR_VERSION_2017_10_10: _ClassVar[Version]
    IR_VERSION_2017_10_30: _ClassVar[Version]
    IR_VERSION_2017_11_3: _ClassVar[Version]
    IR_VERSION_2019_1_22: _ClassVar[Version]
    IR_VERSION_2019_3_18: _ClassVar[Version]
    IR_VERSION_2019_9_19: _ClassVar[Version]
    IR_VERSION_2020_5_8: _ClassVar[Version]
    IR_VERSION_2021_7_30: _ClassVar[Version]
    IR_VERSION_2023_5_5: _ClassVar[Version]
    IR_VERSION_2024_3_25: _ClassVar[Version]
    IR_VERSION_2025_05_12: _ClassVar[Version]
    IR_VERSION_2025_08_26: _ClassVar[Version]
    IR_VERSION: _ClassVar[Version]

class OperatorStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    EXPERIMENTAL: _ClassVar[OperatorStatus]
    STABLE: _ClassVar[OperatorStatus]
_START_VERSION: Version
IR_VERSION_2017_10_10: Version
IR_VERSION_2017_10_30: Version
IR_VERSION_2017_11_3: Version
IR_VERSION_2019_1_22: Version
IR_VERSION_2019_3_18: Version
IR_VERSION_2019_9_19: Version
IR_VERSION_2020_5_8: Version
IR_VERSION_2021_7_30: Version
IR_VERSION_2023_5_5: Version
IR_VERSION_2024_3_25: Version
IR_VERSION_2025_05_12: Version
IR_VERSION_2025_08_26: Version
IR_VERSION: Version
EXPERIMENTAL: OperatorStatus
STABLE: OperatorStatus

class AttributeProto(_message.Message):
    __slots__ = ("name", "ref_attr_name", "doc_string", "type", "f", "i", "s", "t", "g", "sparse_tensor", "tp", "floats", "ints", "strings", "tensors", "graphs", "sparse_tensors", "type_protos")
    class AttributeType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        UNDEFINED: _ClassVar[AttributeProto.AttributeType]
        FLOAT: _ClassVar[AttributeProto.AttributeType]
        INT: _ClassVar[AttributeProto.AttributeType]
        STRING: _ClassVar[AttributeProto.AttributeType]
        TENSOR: _ClassVar[AttributeProto.AttributeType]
        GRAPH: _ClassVar[AttributeProto.AttributeType]
        SPARSE_TENSOR: _ClassVar[AttributeProto.AttributeType]
        TYPE_PROTO: _ClassVar[AttributeProto.AttributeType]
        FLOATS: _ClassVar[AttributeProto.AttributeType]
        INTS: _ClassVar[AttributeProto.AttributeType]
        STRINGS: _ClassVar[AttributeProto.AttributeType]
        TENSORS: _ClassVar[AttributeProto.AttributeType]
        GRAPHS: _ClassVar[AttributeProto.AttributeType]
        SPARSE_TENSORS: _ClassVar[AttributeProto.AttributeType]
        TYPE_PROTOS: _ClassVar[AttributeProto.AttributeType]
    UNDEFINED: AttributeProto.AttributeType
    FLOAT: AttributeProto.AttributeType
    INT: AttributeProto.AttributeType
    STRING: AttributeProto.AttributeType
    TENSOR: AttributeProto.AttributeType
    GRAPH: AttributeProto.AttributeType
    SPARSE_TENSOR: AttributeProto.AttributeType
    TYPE_PROTO: AttributeProto.AttributeType
    FLOATS: AttributeProto.AttributeType
    INTS: AttributeProto.AttributeType
    STRINGS: AttributeProto.AttributeType
    TENSORS: AttributeProto.AttributeType
    GRAPHS: AttributeProto.AttributeType
    SPARSE_TENSORS: AttributeProto.AttributeType
    TYPE_PROTOS: AttributeProto.AttributeType
    NAME_FIELD_NUMBER: _ClassVar[int]
    REF_ATTR_NAME_FIELD_NUMBER: _ClassVar[int]
    DOC_STRING_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    F_FIELD_NUMBER: _ClassVar[int]
    I_FIELD_NUMBER: _ClassVar[int]
    S_FIELD_NUMBER: _ClassVar[int]
    T_FIELD_NUMBER: _ClassVar[int]
    G_FIELD_NUMBER: _ClassVar[int]
    SPARSE_TENSOR_FIELD_NUMBER: _ClassVar[int]
    TP_FIELD_NUMBER: _ClassVar[int]
    FLOATS_FIELD_NUMBER: _ClassVar[int]
    INTS_FIELD_NUMBER: _ClassVar[int]
    STRINGS_FIELD_NUMBER: _ClassVar[int]
    TENSORS_FIELD_NUMBER: _ClassVar[int]
    GRAPHS_FIELD_NUMBER: _ClassVar[int]
    SPARSE_TENSORS_FIELD_NUMBER: _ClassVar[int]
    TYPE_PROTOS_FIELD_NUMBER: _ClassVar[int]
    name: str
    ref_attr_name: str
    doc_string: str
    type: AttributeProto.AttributeType
    f: float
    i: int
    s: bytes
    t: TensorProto
    g: GraphProto
    sparse_tensor: SparseTensorProto
    tp: TypeProto
    floats: _containers.RepeatedScalarFieldContainer[float]
    ints: _containers.RepeatedScalarFieldContainer[int]
    strings: _containers.RepeatedScalarFieldContainer[bytes]
    tensors: _containers.RepeatedCompositeFieldContainer[TensorProto]
    graphs: _containers.RepeatedCompositeFieldContainer[GraphProto]
    sparse_tensors: _containers.RepeatedCompositeFieldContainer[SparseTensorProto]
    type_protos: _containers.RepeatedCompositeFieldContainer[TypeProto]
    def __init__(self, name: _Optional[str] = ..., ref_attr_name: _Optional[str] = ..., doc_string: _Optional[str] = ..., type: _Optional[_Union[AttributeProto.AttributeType, str]] = ..., f: _Optional[float] = ..., i: _Optional[int] = ..., s: _Optional[bytes] = ..., t: _Optional[_Union[TensorProto, _Mapping]] = ..., g: _Optional[_Union[GraphProto, _Mapping]] = ..., sparse_tensor: _Optional[_Union[SparseTensorProto, _Mapping]] = ..., tp: _Optional[_Union[TypeProto, _Mapping]] = ..., floats: _Optional[_Iterable[float]] = ..., ints: _Optional[_Iterable[int]] = ..., strings: _Optional[_Iterable[bytes]] = ..., tensors: _Optional[_Iterable[_Union[TensorProto, _Mapping]]] = ..., graphs: _Optional[_Iterable[_Union[GraphProto, _Mapping]]] = ..., sparse_tensors: _Optional[_Iterable[_Union[SparseTensorProto, _Mapping]]] = ..., type_protos: _Optional[_Iterable[_Union[TypeProto, _Mapping]]] = ...) -> None: ...

class ValueInfoProto(_message.Message):
    __slots__ = ("name", "type", "doc_string", "metadata_props")
    NAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    DOC_STRING_FIELD_NUMBER: _ClassVar[int]
    METADATA_PROPS_FIELD_NUMBER: _ClassVar[int]
    name: str
    type: TypeProto
    doc_string: str
    metadata_props: _containers.RepeatedCompositeFieldContainer[StringStringEntryProto]
    def __init__(self, name: _Optional[str] = ..., type: _Optional[_Union[TypeProto, _Mapping]] = ..., doc_string: _Optional[str] = ..., metadata_props: _Optional[_Iterable[_Union[StringStringEntryProto, _Mapping]]] = ...) -> None: ...

class NodeProto(_message.Message):
    __slots__ = ("input", "output", "name", "op_type", "domain", "overload", "attribute", "doc_string", "metadata_props", "device_configurations")
    INPUT_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    OP_TYPE_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    OVERLOAD_FIELD_NUMBER: _ClassVar[int]
    ATTRIBUTE_FIELD_NUMBER: _ClassVar[int]
    DOC_STRING_FIELD_NUMBER: _ClassVar[int]
    METADATA_PROPS_FIELD_NUMBER: _ClassVar[int]
    DEVICE_CONFIGURATIONS_FIELD_NUMBER: _ClassVar[int]
    input: _containers.RepeatedScalarFieldContainer[str]
    output: _containers.RepeatedScalarFieldContainer[str]
    name: str
    op_type: str
    domain: str
    overload: str
    attribute: _containers.RepeatedCompositeFieldContainer[AttributeProto]
    doc_string: str
    metadata_props: _containers.RepeatedCompositeFieldContainer[StringStringEntryProto]
    device_configurations: _containers.RepeatedCompositeFieldContainer[NodeDeviceConfigurationProto]
    def __init__(self, input: _Optional[_Iterable[str]] = ..., output: _Optional[_Iterable[str]] = ..., name: _Optional[str] = ..., op_type: _Optional[str] = ..., domain: _Optional[str] = ..., overload: _Optional[str] = ..., attribute: _Optional[_Iterable[_Union[AttributeProto, _Mapping]]] = ..., doc_string: _Optional[str] = ..., metadata_props: _Optional[_Iterable[_Union[StringStringEntryProto, _Mapping]]] = ..., device_configurations: _Optional[_Iterable[_Union[NodeDeviceConfigurationProto, _Mapping]]] = ...) -> None: ...

class IntIntListEntryProto(_message.Message):
    __slots__ = ("key", "value")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: int
    value: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, key: _Optional[int] = ..., value: _Optional[_Iterable[int]] = ...) -> None: ...

class NodeDeviceConfigurationProto(_message.Message):
    __slots__ = ("configuration_id", "sharding_spec", "pipeline_stage")
    CONFIGURATION_ID_FIELD_NUMBER: _ClassVar[int]
    SHARDING_SPEC_FIELD_NUMBER: _ClassVar[int]
    PIPELINE_STAGE_FIELD_NUMBER: _ClassVar[int]
    configuration_id: str
    sharding_spec: _containers.RepeatedCompositeFieldContainer[ShardingSpecProto]
    pipeline_stage: int
    def __init__(self, configuration_id: _Optional[str] = ..., sharding_spec: _Optional[_Iterable[_Union[ShardingSpecProto, _Mapping]]] = ..., pipeline_stage: _Optional[int] = ...) -> None: ...

class ShardingSpecProto(_message.Message):
    __slots__ = ("tensor_name", "device", "index_to_device_group_map", "sharded_dim")
    TENSOR_NAME_FIELD_NUMBER: _ClassVar[int]
    DEVICE_FIELD_NUMBER: _ClassVar[int]
    INDEX_TO_DEVICE_GROUP_MAP_FIELD_NUMBER: _ClassVar[int]
    SHARDED_DIM_FIELD_NUMBER: _ClassVar[int]
    tensor_name: str
    device: _containers.RepeatedScalarFieldContainer[int]
    index_to_device_group_map: _containers.RepeatedCompositeFieldContainer[IntIntListEntryProto]
    sharded_dim: _containers.RepeatedCompositeFieldContainer[ShardedDimProto]
    def __init__(self, tensor_name: _Optional[str] = ..., device: _Optional[_Iterable[int]] = ..., index_to_device_group_map: _Optional[_Iterable[_Union[IntIntListEntryProto, _Mapping]]] = ..., sharded_dim: _Optional[_Iterable[_Union[ShardedDimProto, _Mapping]]] = ...) -> None: ...

class ShardedDimProto(_message.Message):
    __slots__ = ("axis", "simple_sharding")
    AXIS_FIELD_NUMBER: _ClassVar[int]
    SIMPLE_SHARDING_FIELD_NUMBER: _ClassVar[int]
    axis: int
    simple_sharding: _containers.RepeatedCompositeFieldContainer[SimpleShardedDimProto]
    def __init__(self, axis: _Optional[int] = ..., simple_sharding: _Optional[_Iterable[_Union[SimpleShardedDimProto, _Mapping]]] = ...) -> None: ...

class SimpleShardedDimProto(_message.Message):
    __slots__ = ("dim_value", "dim_param", "num_shards")
    DIM_VALUE_FIELD_NUMBER: _ClassVar[int]
    DIM_PARAM_FIELD_NUMBER: _ClassVar[int]
    NUM_SHARDS_FIELD_NUMBER: _ClassVar[int]
    dim_value: int
    dim_param: str
    num_shards: int
    def __init__(self, dim_value: _Optional[int] = ..., dim_param: _Optional[str] = ..., num_shards: _Optional[int] = ...) -> None: ...

class TrainingInfoProto(_message.Message):
    __slots__ = ("initialization", "algorithm", "initialization_binding", "update_binding")
    INITIALIZATION_FIELD_NUMBER: _ClassVar[int]
    ALGORITHM_FIELD_NUMBER: _ClassVar[int]
    INITIALIZATION_BINDING_FIELD_NUMBER: _ClassVar[int]
    UPDATE_BINDING_FIELD_NUMBER: _ClassVar[int]
    initialization: GraphProto
    algorithm: GraphProto
    initialization_binding: _containers.RepeatedCompositeFieldContainer[StringStringEntryProto]
    update_binding: _containers.RepeatedCompositeFieldContainer[StringStringEntryProto]
    def __init__(self, initialization: _Optional[_Union[GraphProto, _Mapping]] = ..., algorithm: _Optional[_Union[GraphProto, _Mapping]] = ..., initialization_binding: _Optional[_Iterable[_Union[StringStringEntryProto, _Mapping]]] = ..., update_binding: _Optional[_Iterable[_Union[StringStringEntryProto, _Mapping]]] = ...) -> None: ...

class ModelProto(_message.Message):
    __slots__ = ("ir_version", "opset_import", "producer_name", "producer_version", "domain", "model_version", "doc_string", "graph", "metadata_props", "training_info", "functions", "configuration")
    IR_VERSION_FIELD_NUMBER: _ClassVar[int]
    OPSET_IMPORT_FIELD_NUMBER: _ClassVar[int]
    PRODUCER_NAME_FIELD_NUMBER: _ClassVar[int]
    PRODUCER_VERSION_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    MODEL_VERSION_FIELD_NUMBER: _ClassVar[int]
    DOC_STRING_FIELD_NUMBER: _ClassVar[int]
    GRAPH_FIELD_NUMBER: _ClassVar[int]
    METADATA_PROPS_FIELD_NUMBER: _ClassVar[int]
    TRAINING_INFO_FIELD_NUMBER: _ClassVar[int]
    FUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    CONFIGURATION_FIELD_NUMBER: _ClassVar[int]
    ir_version: int
    opset_import: _containers.RepeatedCompositeFieldContainer[OperatorSetIdProto]
    producer_name: str
    producer_version: str
    domain: str
    model_version: int
    doc_string: str
    graph: GraphProto
    metadata_props: _containers.RepeatedCompositeFieldContainer[StringStringEntryProto]
    training_info: _containers.RepeatedCompositeFieldContainer[TrainingInfoProto]
    functions: _containers.RepeatedCompositeFieldContainer[FunctionProto]
    configuration: _containers.RepeatedCompositeFieldContainer[DeviceConfigurationProto]
    def __init__(self, ir_version: _Optional[int] = ..., opset_import: _Optional[_Iterable[_Union[OperatorSetIdProto, _Mapping]]] = ..., producer_name: _Optional[str] = ..., producer_version: _Optional[str] = ..., domain: _Optional[str] = ..., model_version: _Optional[int] = ..., doc_string: _Optional[str] = ..., graph: _Optional[_Union[GraphProto, _Mapping]] = ..., metadata_props: _Optional[_Iterable[_Union[StringStringEntryProto, _Mapping]]] = ..., training_info: _Optional[_Iterable[_Union[TrainingInfoProto, _Mapping]]] = ..., functions: _Optional[_Iterable[_Union[FunctionProto, _Mapping]]] = ..., configuration: _Optional[_Iterable[_Union[DeviceConfigurationProto, _Mapping]]] = ...) -> None: ...

class DeviceConfigurationProto(_message.Message):
    __slots__ = ("name", "num_devices", "device")
    NAME_FIELD_NUMBER: _ClassVar[int]
    NUM_DEVICES_FIELD_NUMBER: _ClassVar[int]
    DEVICE_FIELD_NUMBER: _ClassVar[int]
    name: str
    num_devices: int
    device: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, name: _Optional[str] = ..., num_devices: _Optional[int] = ..., device: _Optional[_Iterable[str]] = ...) -> None: ...

class StringStringEntryProto(_message.Message):
    __slots__ = ("key", "value")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: str
    def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

class TensorAnnotation(_message.Message):
    __slots__ = ("tensor_name", "quant_parameter_tensor_names")
    TENSOR_NAME_FIELD_NUMBER: _ClassVar[int]
    QUANT_PARAMETER_TENSOR_NAMES_FIELD_NUMBER: _ClassVar[int]
    tensor_name: str
    quant_parameter_tensor_names: _containers.RepeatedCompositeFieldContainer[StringStringEntryProto]
    def __init__(self, tensor_name: _Optional[str] = ..., quant_parameter_tensor_names: _Optional[_Iterable[_Union[StringStringEntryProto, _Mapping]]] = ...) -> None: ...

class GraphProto(_message.Message):
    __slots__ = ("node", "name", "initializer", "sparse_initializer", "doc_string", "input", "output", "value_info", "quantization_annotation", "metadata_props")
    NODE_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    INITIALIZER_FIELD_NUMBER: _ClassVar[int]
    SPARSE_INITIALIZER_FIELD_NUMBER: _ClassVar[int]
    DOC_STRING_FIELD_NUMBER: _ClassVar[int]
    INPUT_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_FIELD_NUMBER: _ClassVar[int]
    VALUE_INFO_FIELD_NUMBER: _ClassVar[int]
    QUANTIZATION_ANNOTATION_FIELD_NUMBER: _ClassVar[int]
    METADATA_PROPS_FIELD_NUMBER: _ClassVar[int]
    node: _containers.RepeatedCompositeFieldContainer[NodeProto]
    name: str
    initializer: _containers.RepeatedCompositeFieldContainer[TensorProto]
    sparse_initializer: _containers.RepeatedCompositeFieldContainer[SparseTensorProto]
    doc_string: str
    input: _containers.RepeatedCompositeFieldContainer[ValueInfoProto]
    output: _containers.RepeatedCompositeFieldContainer[ValueInfoProto]
    value_info: _containers.RepeatedCompositeFieldContainer[ValueInfoProto]
    quantization_annotation: _containers.RepeatedCompositeFieldContainer[TensorAnnotation]
    metadata_props: _containers.RepeatedCompositeFieldContainer[StringStringEntryProto]
    def __init__(self, node: _Optional[_Iterable[_Union[NodeProto, _Mapping]]] = ..., name: _Optional[str] = ..., initializer: _Optional[_Iterable[_Union[TensorProto, _Mapping]]] = ..., sparse_initializer: _Optional[_Iterable[_Union[SparseTensorProto, _Mapping]]] = ..., doc_string: _Optional[str] = ..., input: _Optional[_Iterable[_Union[ValueInfoProto, _Mapping]]] = ..., output: _Optional[_Iterable[_Union[ValueInfoProto, _Mapping]]] = ..., value_info: _Optional[_Iterable[_Union[ValueInfoProto, _Mapping]]] = ..., quantization_annotation: _Optional[_Iterable[_Union[TensorAnnotation, _Mapping]]] = ..., metadata_props: _Optional[_Iterable[_Union[StringStringEntryProto, _Mapping]]] = ...) -> None: ...

class TensorProto(_message.Message):
    __slots__ = ("dims", "data_type", "segment", "float_data", "int32_data", "string_data", "int64_data", "name", "doc_string", "raw_data", "external_data", "data_location", "double_data", "uint64_data", "metadata_props")
    class DataType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        UNDEFINED: _ClassVar[TensorProto.DataType]
        FLOAT: _ClassVar[TensorProto.DataType]
        UINT8: _ClassVar[TensorProto.DataType]
        INT8: _ClassVar[TensorProto.DataType]
        UINT16: _ClassVar[TensorProto.DataType]
        INT16: _ClassVar[TensorProto.DataType]
        INT32: _ClassVar[TensorProto.DataType]
        INT64: _ClassVar[TensorProto.DataType]
        STRING: _ClassVar[TensorProto.DataType]
        BOOL: _ClassVar[TensorProto.DataType]
        FLOAT16: _ClassVar[TensorProto.DataType]
        DOUBLE: _ClassVar[TensorProto.DataType]
        UINT32: _ClassVar[TensorProto.DataType]
        UINT64: _ClassVar[TensorProto.DataType]
        COMPLEX64: _ClassVar[TensorProto.DataType]
        COMPLEX128: _ClassVar[TensorProto.DataType]
        BFLOAT16: _ClassVar[TensorProto.DataType]
        FLOAT8E4M3FN: _ClassVar[TensorProto.DataType]
        FLOAT8E4M3FNUZ: _ClassVar[TensorProto.DataType]
        FLOAT8E5M2: _ClassVar[TensorProto.DataType]
        FLOAT8E5M2FNUZ: _ClassVar[TensorProto.DataType]
        UINT4: _ClassVar[TensorProto.DataType]
        INT4: _ClassVar[TensorProto.DataType]
        FLOAT4E2M1: _ClassVar[TensorProto.DataType]
        FLOAT8E8M0: _ClassVar[TensorProto.DataType]
        UINT2: _ClassVar[TensorProto.DataType]
        INT2: _ClassVar[TensorProto.DataType]
    UNDEFINED: TensorProto.DataType
    FLOAT: TensorProto.DataType
    UINT8: TensorProto.DataType
    INT8: TensorProto.DataType
    UINT16: TensorProto.DataType
    INT16: TensorProto.DataType
    INT32: TensorProto.DataType
    INT64: TensorProto.DataType
    STRING: TensorProto.DataType
    BOOL: TensorProto.DataType
    FLOAT16: TensorProto.DataType
    DOUBLE: TensorProto.DataType
    UINT32: TensorProto.DataType
    UINT64: TensorProto.DataType
    COMPLEX64: TensorProto.DataType
    COMPLEX128: TensorProto.DataType
    BFLOAT16: TensorProto.DataType
    FLOAT8E4M3FN: TensorProto.DataType
    FLOAT8E4M3FNUZ: TensorProto.DataType
    FLOAT8E5M2: TensorProto.DataType
    FLOAT8E5M2FNUZ: TensorProto.DataType
    UINT4: TensorProto.DataType
    INT4: TensorProto.DataType
    FLOAT4E2M1: TensorProto.DataType
    FLOAT8E8M0: TensorProto.DataType
    UINT2: TensorProto.DataType
    INT2: TensorProto.DataType
    class DataLocation(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        DEFAULT: _ClassVar[TensorProto.DataLocation]
        EXTERNAL: _ClassVar[TensorProto.DataLocation]
    DEFAULT: TensorProto.DataLocation
    EXTERNAL: TensorProto.DataLocation
    class Segment(_message.Message):
        __slots__ = ("begin", "end")
        BEGIN_FIELD_NUMBER: _ClassVar[int]
        END_FIELD_NUMBER: _ClassVar[int]
        begin: int
        end: int
        def __init__(self, begin: _Optional[int] = ..., end: _Optional[int] = ...) -> None: ...
    DIMS_FIELD_NUMBER: _ClassVar[int]
    DATA_TYPE_FIELD_NUMBER: _ClassVar[int]
    SEGMENT_FIELD_NUMBER: _ClassVar[int]
    FLOAT_DATA_FIELD_NUMBER: _ClassVar[int]
    INT32_DATA_FIELD_NUMBER: _ClassVar[int]
    STRING_DATA_FIELD_NUMBER: _ClassVar[int]
    INT64_DATA_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    DOC_STRING_FIELD_NUMBER: _ClassVar[int]
    RAW_DATA_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_DATA_FIELD_NUMBER: _ClassVar[int]
    DATA_LOCATION_FIELD_NUMBER: _ClassVar[int]
    DOUBLE_DATA_FIELD_NUMBER: _ClassVar[int]
    UINT64_DATA_FIELD_NUMBER: _ClassVar[int]
    METADATA_PROPS_FIELD_NUMBER: _ClassVar[int]
    dims: _containers.RepeatedScalarFieldContainer[int]
    data_type: int
    segment: TensorProto.Segment
    float_data: _containers.RepeatedScalarFieldContainer[float]
    int32_data: _containers.RepeatedScalarFieldContainer[int]
    string_data: _containers.RepeatedScalarFieldContainer[bytes]
    int64_data: _containers.RepeatedScalarFieldContainer[int]
    name: str
    doc_string: str
    raw_data: bytes
    external_data: _containers.RepeatedCompositeFieldContainer[StringStringEntryProto]
    data_location: TensorProto.DataLocation
    double_data: _containers.RepeatedScalarFieldContainer[float]
    uint64_data: _containers.RepeatedScalarFieldContainer[int]
    metadata_props: _containers.RepeatedCompositeFieldContainer[StringStringEntryProto]
    def __init__(self, dims: _Optional[_Iterable[int]] = ..., data_type: _Optional[int] = ..., segment: _Optional[_Union[TensorProto.Segment, _Mapping]] = ..., float_data: _Optional[_Iterable[float]] = ..., int32_data: _Optional[_Iterable[int]] = ..., string_data: _Optional[_Iterable[bytes]] = ..., int64_data: _Optional[_Iterable[int]] = ..., name: _Optional[str] = ..., doc_string: _Optional[str] = ..., raw_data: _Optional[bytes] = ..., external_data: _Optional[_Iterable[_Union[StringStringEntryProto, _Mapping]]] = ..., data_location: _Optional[_Union[TensorProto.DataLocation, str]] = ..., double_data: _Optional[_Iterable[float]] = ..., uint64_data: _Optional[_Iterable[int]] = ..., metadata_props: _Optional[_Iterable[_Union[StringStringEntryProto, _Mapping]]] = ...) -> None: ...

class SparseTensorProto(_message.Message):
    __slots__ = ("values", "indices", "dims")
    VALUES_FIELD_NUMBER: _ClassVar[int]
    INDICES_FIELD_NUMBER: _ClassVar[int]
    DIMS_FIELD_NUMBER: _ClassVar[int]
    values: TensorProto
    indices: TensorProto
    dims: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, values: _Optional[_Union[TensorProto, _Mapping]] = ..., indices: _Optional[_Union[TensorProto, _Mapping]] = ..., dims: _Optional[_Iterable[int]] = ...) -> None: ...

class TensorShapeProto(_message.Message):
    __slots__ = ("dim",)
    class Dimension(_message.Message):
        __slots__ = ("dim_value", "dim_param", "denotation")
        DIM_VALUE_FIELD_NUMBER: _ClassVar[int]
        DIM_PARAM_FIELD_NUMBER: _ClassVar[int]
        DENOTATION_FIELD_NUMBER: _ClassVar[int]
        dim_value: int
        dim_param: str
        denotation: str
        def __init__(self, dim_value: _Optional[int] = ..., dim_param: _Optional[str] = ..., denotation: _Optional[str] = ...) -> None: ...
    DIM_FIELD_NUMBER: _ClassVar[int]
    dim: _containers.RepeatedCompositeFieldContainer[TensorShapeProto.Dimension]
    def __init__(self, dim: _Optional[_Iterable[_Union[TensorShapeProto.Dimension, _Mapping]]] = ...) -> None: ...

class TypeProto(_message.Message):
    __slots__ = ("tensor_type", "sequence_type", "map_type", "optional_type", "sparse_tensor_type", "opaque_type", "denotation")
    class Tensor(_message.Message):
        __slots__ = ("elem_type", "shape")
        ELEM_TYPE_FIELD_NUMBER: _ClassVar[int]
        SHAPE_FIELD_NUMBER: _ClassVar[int]
        elem_type: int
        shape: TensorShapeProto
        def __init__(self, elem_type: _Optional[int] = ..., shape: _Optional[_Union[TensorShapeProto, _Mapping]] = ...) -> None: ...
    class Sequence(_message.Message):
        __slots__ = ("elem_type",)
        ELEM_TYPE_FIELD_NUMBER: _ClassVar[int]
        elem_type: TypeProto
        def __init__(self, elem_type: _Optional[_Union[TypeProto, _Mapping]] = ...) -> None: ...
    class Map(_message.Message):
        __slots__ = ("key_type", "value_type")
        KEY_TYPE_FIELD_NUMBER: _ClassVar[int]
        VALUE_TYPE_FIELD_NUMBER: _ClassVar[int]
        key_type: int
        value_type: TypeProto
        def __init__(self, key_type: _Optional[int] = ..., value_type: _Optional[_Union[TypeProto, _Mapping]] = ...) -> None: ...
    class Optional(_message.Message):
        __slots__ = ("elem_type",)
        ELEM_TYPE_FIELD_NUMBER: _ClassVar[int]
        elem_type: TypeProto
        def __init__(self, elem_type: _Optional[_Union[TypeProto, _Mapping]] = ...) -> None: ...
    class SparseTensor(_message.Message):
        __slots__ = ("elem_type", "shape")
        ELEM_TYPE_FIELD_NUMBER: _ClassVar[int]
        SHAPE_FIELD_NUMBER: _ClassVar[int]
        elem_type: int
        shape: TensorShapeProto
        def __init__(self, elem_type: _Optional[int] = ..., shape: _Optional[_Union[TensorShapeProto, _Mapping]] = ...) -> None: ...
    class Opaque(_message.Message):
        __slots__ = ("domain", "name")
        DOMAIN_FIELD_NUMBER: _ClassVar[int]
        NAME_FIELD_NUMBER: _ClassVar[int]
        domain: str
        name: str
        def __init__(self, domain: _Optional[str] = ..., name: _Optional[str] = ...) -> None: ...
    TENSOR_TYPE_FIELD_NUMBER: _ClassVar[int]
    SEQUENCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    MAP_TYPE_FIELD_NUMBER: _ClassVar[int]
    OPTIONAL_TYPE_FIELD_NUMBER: _ClassVar[int]
    SPARSE_TENSOR_TYPE_FIELD_NUMBER: _ClassVar[int]
    OPAQUE_TYPE_FIELD_NUMBER: _ClassVar[int]
    DENOTATION_FIELD_NUMBER: _ClassVar[int]
    tensor_type: TypeProto.Tensor
    sequence_type: TypeProto.Sequence
    map_type: TypeProto.Map
    optional_type: TypeProto.Optional
    sparse_tensor_type: TypeProto.SparseTensor
    opaque_type: TypeProto.Opaque
    denotation: str
    def __init__(self, tensor_type: _Optional[_Union[TypeProto.Tensor, _Mapping]] = ..., sequence_type: _Optional[_Union[TypeProto.Sequence, _Mapping]] = ..., map_type: _Optional[_Union[TypeProto.Map, _Mapping]] = ..., optional_type: _Optional[_Union[TypeProto.Optional, _Mapping]] = ..., sparse_tensor_type: _Optional[_Union[TypeProto.SparseTensor, _Mapping]] = ..., opaque_type: _Optional[_Union[TypeProto.Opaque, _Mapping]] = ..., denotation: _Optional[str] = ...) -> None: ...

class OperatorSetIdProto(_message.Message):
    __slots__ = ("domain", "version")
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    domain: str
    version: int
    def __init__(self, domain: _Optional[str] = ..., version: _Optional[int] = ...) -> None: ...

class FunctionProto(_message.Message):
    __slots__ = ("name", "input", "output", "attribute", "attribute_proto", "node", "doc_string", "opset_import", "domain", "overload", "value_info", "metadata_props")
    NAME_FIELD_NUMBER: _ClassVar[int]
    INPUT_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_FIELD_NUMBER: _ClassVar[int]
    ATTRIBUTE_FIELD_NUMBER: _ClassVar[int]
    ATTRIBUTE_PROTO_FIELD_NUMBER: _ClassVar[int]
    NODE_FIELD_NUMBER: _ClassVar[int]
    DOC_STRING_FIELD_NUMBER: _ClassVar[int]
    OPSET_IMPORT_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    OVERLOAD_FIELD_NUMBER: _ClassVar[int]
    VALUE_INFO_FIELD_NUMBER: _ClassVar[int]
    METADATA_PROPS_FIELD_NUMBER: _ClassVar[int]
    name: str
    input: _containers.RepeatedScalarFieldContainer[str]
    output: _containers.RepeatedScalarFieldContainer[str]
    attribute: _containers.RepeatedScalarFieldContainer[str]
    attribute_proto: _containers.RepeatedCompositeFieldContainer[AttributeProto]
    node: _containers.RepeatedCompositeFieldContainer[NodeProto]
    doc_string: str
    opset_import: _containers.RepeatedCompositeFieldContainer[OperatorSetIdProto]
    domain: str
    overload: str
    value_info: _containers.RepeatedCompositeFieldContainer[ValueInfoProto]
    metadata_props: _containers.RepeatedCompositeFieldContainer[StringStringEntryProto]
    def __init__(self, name: _Optional[str] = ..., input: _Optional[_Iterable[str]] = ..., output: _Optional[_Iterable[str]] = ..., attribute: _Optional[_Iterable[str]] = ..., attribute_proto: _Optional[_Iterable[_Union[AttributeProto, _Mapping]]] = ..., node: _Optional[_Iterable[_Union[NodeProto, _Mapping]]] = ..., doc_string: _Optional[str] = ..., opset_import: _Optional[_Iterable[_Union[OperatorSetIdProto, _Mapping]]] = ..., domain: _Optional[str] = ..., overload: _Optional[str] = ..., value_info: _Optional[_Iterable[_Union[ValueInfoProto, _Mapping]]] = ..., metadata_props: _Optional[_Iterable[_Union[StringStringEntryProto, _Mapping]]] = ...) -> None: ...



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect


def compute_if_outputs(x, cond):
    if cond:
        return []
    return x


class If(Base):
    @staticmethod
    def export_if() -> None:
        # Given a bool scalar input cond.
        # return constant tensor x if cond is True, otherwise return constant tensor y.

        then_out = onnx.helper.make_tensor_value_info(
            "then_out", onnx.TensorProto.FLOAT, [5]
        )
        else_out = onnx.helper.make_tensor_value_info(
            "else_out", onnx.TensorProto.FLOAT, [5]
        )

        x = np.array([1, 2, 3, 4, 5]).astype(np.float32)
        y = np.array([5, 4, 3, 2, 1]).astype(np.float32)

        then_const_node = onnx.helper.make_node(
            "Constant",
            inputs=[],
            outputs=["then_out"],
            value=onnx.numpy_helper.from_array(x),
        )

        else_const_node = onnx.helper.make_node(
            "Constant",
            inputs=[],
            outputs=["else_out"],
            value=onnx.numpy_helper.from_array(y),
        )

        then_body = onnx.helper.make_graph(
            [then_const_node], "then_body", [], [then_out]
        )

        else_body = onnx.helper.make_graph(
            [else_const_node], "else_body", [], [else_out]
        )

        if_node = onnx.helper.make_node(
            "If",
            inputs=["cond"],
            outputs=["res"],
            then_branch=then_body,
            else_branch=else_body,
        )

        cond = np.array(1).astype(bool)
        res = x if cond else y
        expect(
            if_node,
            inputs=[cond],
            outputs=[res],
            name="test_if",
            opset_imports=[onnx.helper.make_opsetid("", 11)],
        )

    @staticmethod
    def export_if_seq() -> None:
        # Given a bool scalar input cond.
        # return constant sequence x if cond is True, otherwise return constant sequence y.

        then_out = onnx.helper.make_tensor_sequence_value_info(
            "then_out", onnx.TensorProto.FLOAT, shape=[5]
        )
        else_out = onnx.helper.make_tensor_sequence_value_info(
            "else_out", onnx.TensorProto.FLOAT, shape=[5]
        )

        x = [np.array([1, 2, 3, 4, 5]).astype(np.float32)]
        y = [np.array([5, 4, 3, 2, 1]).astype(np.float32)]

        then_const_node = onnx.helper.make_node(
            "Constant",
            inputs=[],
            outputs=["x"],
            value=onnx.numpy_helper.from_array(x[0]),
        )

        then_seq_node = onnx.helper.make_node(
            "SequenceConstruct", inputs=["x"], outputs=["then_out"]
        )

        else_const_node = onnx.helper.make_node(
            "Constant",
            inputs=[],
            outputs=["y"],
            value=onnx.numpy_helper.from_array(y[0]),
        )

        else_seq_node = onnx.helper.make_node(
            "SequenceConstruct", inputs=["y"], outputs=["else_out"]
        )

        then_body = onnx.helper.make_graph(
            [then_const_node, then_seq_node], "then_body", [], [then_out]
        )

        else_body = onnx.helper.make_graph(
            [else_const_node, else_seq_node], "else_body", [], [else_out]
        )

        if_node = onnx.helper.make_node(
            "If",
            inputs=["cond"],
            outputs=["res"],
            then_branch=then_body,
            else_branch=else_body,
        )

        cond = np.array(1).astype(bool)
        res = x if cond else y
        expect(
            if_node,
            inputs=[cond],
            outputs=[res],
            name="test_if_seq",
            opset_imports=[onnx.helper.make_opsetid("", 13)],
        )

    @staticmethod
    def export_if_optional() -> None:
        # Given a bool scalar input cond, return an empty optional sequence of
        # tensor if True, return an optional sequence with value x
        # (the input optional sequence) otherwise.

        ten_in_tp = onnx.helper.make_tensor_type_proto(
            onnx.TensorProto.FLOAT, shape=[5]
        )
        seq_in_tp = onnx.helper.make_sequence_type_proto(ten_in_tp)

        then_out_tensor_tp = onnx.helper.make_tensor_type_proto(
            onnx.TensorProto.FLOAT, shape=[5]
        )
        then_out_seq_tp = onnx.helper.make_sequence_type_proto(then_out_tensor_tp)
        then_out_opt_tp = onnx.helper.make_optional_type_proto(then_out_seq_tp)
        then_out = onnx.helper.make_value_info("optional_empty", then_out_opt_tp)

        else_out_tensor_tp = onnx.helper.make_tensor_type_proto(
            onnx.TensorProto.FLOAT, shape=[5]
        )
        else_out_seq_tp = onnx.helper.make_sequence_type_proto(else_out_tensor_tp)
        else_out_opt_tp = onnx.helper.make_optional_type_proto(else_out_seq_tp)
        else_out = onnx.helper.make_value_info("else_opt", else_out_opt_tp)

        x = [np.array([1, 2, 3, 4, 5]).astype(np.float32)]
        cond = np.array(0).astype(bool)
        res = compute_if_outputs(x, cond)

        opt_empty_in = onnx.helper.make_node(
            "Optional", inputs=[], outputs=["optional_empty"], type=seq_in_tp
        )

        then_body = onnx.helper.make_graph([opt_empty_in], "then_body", [], [then_out])

        else_const_node = onnx.helper.make_node(
            "Constant",
            inputs=[],
            outputs=["x"],
            value=onnx.numpy_helper.from_array(x[0]),
        )

        else_seq_node = onnx.helper.make_node(
            "SequenceConstruct", inputs=["x"], outputs=["else_seq"]
        )

        else_optional_seq_node = onnx.helper.make_node(
            "Optional", inputs=["else_seq"], outputs=["else_opt"]
        )

        else_body = onnx.helper.make_graph(
            [else_const_node, else_seq_node, else_optional_seq_node],
            "else_body",
            [],
            [else_out],
        )

        if_node = onnx.helper.make_node(
            "If",
            inputs=["cond"],
            outputs=["sequence"],
            then_branch=then_body,
            else_branch=else_body,
        )

        expect(
            if_node,
            inputs=[cond],
            outputs=[res],
            name="test_if_opt",
            output_type_protos=[else_out_opt_tp],
            opset_imports=[onnx.helper.make_opsetid("", 16)],
        )



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect
from onnx.reference.ops.op_rotary_embedding import rotary_embedding


class RotaryEmbedding(Base):
    @staticmethod
    def export_rotary_embedding() -> None:
        node = onnx.helper.make_node(
            "RotaryEmbedding",
            inputs=["input", "cos_cache", "sin_cache", "position_ids"],
            outputs=["output"],
        )

        input_data = np.random.rand(2, 4, 3, 8).astype(np.float32)
        position_ids_data = np.random.uniform(0, 50, (2, 3)).astype(np.int64)
        sin_cache_data = np.random.rand(50, 4).astype(np.float32)
        cos_cache_data = np.random.rand(50, 4).astype(np.float32)

        expected_output = rotary_embedding(
            input_data, cos_cache_data, sin_cache_data, position_ids=position_ids_data
        )

        expect(
            node,
            inputs=[input_data, cos_cache_data, sin_cache_data, position_ids_data],
            outputs=[expected_output],
            name="test_rotary_embedding",
        )

    @staticmethod
    def export_rotary_embedding_3d_input() -> None:
        num_heads = 4
        node = onnx.helper.make_node(
            "RotaryEmbedding",
            inputs=["input", "cos_cache", "sin_cache", "position_ids"],
            outputs=["output"],
            num_heads=num_heads,
        )

        input_data = np.random.rand(2, 3, 32).astype(np.float32)
        position_ids_data = np.random.uniform(0, 50, (2, 3)).astype(np.int64)
        sin_cache_data = np.random.rand(50, 4).astype(np.float32)
        cos_cache_data = np.random.rand(50, 4).astype(np.float32)

        expected_output = rotary_embedding(
            input_data,
            cos_cache_data,
            sin_cache_data,
            position_ids=position_ids_data,
            num_heads=num_heads,
        )

        expect(
            node,
            inputs=[input_data, cos_cache_data, sin_cache_data, position_ids_data],
            outputs=[expected_output],
            name="test_rotary_embedding_3d_input",
        )

    @staticmethod
    def export_rotary_embedding_interleaved() -> None:
        node = onnx.helper.make_node(
            "RotaryEmbedding",
            inputs=["input", "cos_cache", "sin_cache", "position_ids"],
            outputs=["output"],
            interleaved=1,
        )

        input_data = np.random.rand(2, 4, 3, 8).astype(np.float32)
        position_ids_data = np.random.uniform(0, 50, (2, 3)).astype(np.int64)
        sin_cache_data = np.random.rand(50, 4).astype(np.float32)
        cos_cache_data = np.random.rand(50, 4).astype(np.float32)

        expected_output = rotary_embedding(
            input_data,
            cos_cache_data,
            sin_cache_data,
            position_ids=position_ids_data,
            interleaved=1,
        )

        expect(
            node,
            inputs=[input_data, cos_cache_data, sin_cache_data, position_ids_data],
            outputs=[expected_output],
            name="test_rotary_embedding_interleaved",
        )

    @staticmethod
    def export_rotary_embedding_with_rotary_dim() -> None:
        node = onnx.helper.make_node(
            "RotaryEmbedding",
            inputs=["input", "cos_cache", "sin_cache", "position_ids"],
            outputs=["output"],
            rotary_embedding_dim=4,
        )

        input_data = np.random.rand(2, 4, 3, 8).astype(np.float32)
        position_ids_data = np.random.uniform(0, 50, (2, 3)).astype(np.int64)
        sin_cache_data = np.random.rand(50, 2).astype(np.float32)
        cos_cache_data = np.random.rand(50, 2).astype(np.float32)

        expected_output = rotary_embedding(
            input_data,
            cos_cache_data,
            sin_cache_data,
            position_ids=position_ids_data,
            rotary_embedding_dim=4,
        )

        expect(
            node,
            inputs=[input_data, cos_cache_data, sin_cache_data, position_ids_data],
            outputs=[expected_output],
            name="test_rotary_embedding_with_rotary_dim",
        )

    @staticmethod
    def export_rotary_embedding_with_interleaved_rotary_dim() -> None:
        node = onnx.helper.make_node(
            "RotaryEmbedding",
            inputs=["input", "cos_cache", "sin_cache", "position_ids"],
            outputs=["output"],
            rotary_embedding_dim=4,
            interleaved=1,
        )

        input_data = np.random.rand(2, 4, 3, 8).astype(np.float32)
        position_ids_data = np.random.uniform(0, 50, (2, 3)).astype(np.int64)
        sin_cache_data = np.random.rand(50, 2).astype(np.float32)
        cos_cache_data = np.random.rand(50, 2).astype(np.float32)

        expected_output = rotary_embedding(
            input_data,
            cos_cache_data,
            sin_cache_data,
            position_ids=position_ids_data,
            interleaved=1,
            rotary_embedding_dim=4,
        )

        expect(
            node,
            inputs=[input_data, cos_cache_data, sin_cache_data, position_ids_data],
            outputs=[expected_output],
            name="test_rotary_embedding_with_interleaved_rotary_dim",
        )

    @staticmethod
    def export_rotary_embedding_no_position_ids() -> None:
        node = onnx.helper.make_node(
            "RotaryEmbedding",
            inputs=["input", "cos_cache", "sin_cache"],
            outputs=["output"],
        )

        input_data = np.random.rand(2, 4, 3, 8).astype(np.float32)
        sin_cache_data = np.random.rand(2, 3, 4).astype(np.float32)
        cos_cache_data = np.random.rand(2, 3, 4).astype(np.float32)

        expected_output = rotary_embedding(input_data, cos_cache_data, sin_cache_data)

        expect(
            node,
            inputs=[input_data, cos_cache_data, sin_cache_data],
            outputs=[expected_output],
            name="test_rotary_embedding_no_position_ids",
        )

    @staticmethod
    def export_rotary_embedding_no_position_ids_interleaved() -> None:
        node = onnx.helper.make_node(
            "RotaryEmbedding",
            inputs=["input", "cos_cache", "sin_cache"],
            outputs=["output"],
            interleaved=1,
        )

        input_data = np.random.rand(2, 4, 3, 8).astype(np.float32)
        sin_cache_data = np.random.rand(2, 3, 4).astype(np.float32)
        cos_cache_data = np.random.rand(2, 3, 4).astype(np.float32)

        expected_output = rotary_embedding(
            input_data,
            cos_cache_data,
            sin_cache_data,
            interleaved=1,
        )

        expect(
            node,
            inputs=[input_data, cos_cache_data, sin_cache_data],
            outputs=[expected_output],
            name="test_rotary_embedding_no_position_ids_interleaved",
        )

    @staticmethod
    def export_rotary_embedding_no_position_ids_rotary_dim() -> None:
        node = onnx.helper.make_node(
            "RotaryEmbedding",
            inputs=["input", "cos_cache", "sin_cache"],
            outputs=["output"],
            rotary_embedding_dim=4,
        )

        input_data = np.random.rand(2, 4, 3, 8).astype(np.float32)
        sin_cache_data = np.random.rand(2, 3, 2).astype(np.float32)
        cos_cache_data = np.random.rand(2, 3, 2).astype(np.float32)

        expected_output = rotary_embedding(
            input_data,
            cos_cache_data,
            sin_cache_data,
            rotary_embedding_dim=4,
        )

        expect(
            node,
            inputs=[input_data, cos_cache_data, sin_cache_data],
            outputs=[expected_output],
            name="test_rotary_embedding_no_position_ids_rotary_dim",
        )



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect


class BitCast(Base):
    @staticmethod
    def export_bitcast_float32_to_int32() -> None:
        """Test bitcasting from float32 to int32 (same size)."""
        node = onnx.helper.make_node(
            "BitCast",
            inputs=["x"],
            outputs=["y"],
            to=onnx.TensorProto.INT32,
        )
        x = np.array([1.0, -2.5, 3.75], dtype=np.float32)
        y = x.view(np.int32)
        expect(node, inputs=[x], outputs=[y], name="test_bitcast_float32_to_int32")

    @staticmethod
    def export_bitcast_int32_to_float32() -> None:
        """Test bitcasting from int32 to float32 (same size)."""
        node = onnx.helper.make_node(
            "BitCast",
            inputs=["x"],
            outputs=["y"],
            to=onnx.TensorProto.FLOAT,
        )
        x = np.array([1065353216, -1071644672, 1081081856], dtype=np.int32)
        y = x.view(np.float32)
        expect(node, inputs=[x], outputs=[y], name="test_bitcast_int32_to_float32")

    @staticmethod
    def export_bitcast_float64_to_int64() -> None:
        """Test bitcasting from float64 to int64 (same size)."""
        node = onnx.helper.make_node(
            "BitCast",
            inputs=["x"],
            outputs=["y"],
            to=onnx.TensorProto.INT64,
        )
        x = np.array([1.0, -2.5, 3.75], dtype=np.float64)
        y = x.view(np.int64)
        expect(node, inputs=[x], outputs=[y], name="test_bitcast_float64_to_int64")

    @staticmethod
    def export_bitcast_int64_to_float64() -> None:
        """Test bitcasting from int64 to float64 (same size)."""
        node = onnx.helper.make_node(
            "BitCast",
            inputs=["x"],
            outputs=["y"],
            to=onnx.TensorProto.DOUBLE,
        )
        x = np.array(
            [4607182418800017408, -4611686018427387904, 4614256656552045184],
            dtype=np.int64,
        )
        y = x.view(np.float64)
        expect(node, inputs=[x], outputs=[y], name="test_bitcast_int64_to_float64")

    @staticmethod
    def export_bitcast_uint32_to_int32() -> None:
        """Test bitcasting from uint32 to int32 (same size, different signedness)."""
        node = onnx.helper.make_node(
            "BitCast",
            inputs=["x"],
            outputs=["y"],
            to=onnx.TensorProto.INT32,
        )
        x = np.array([4294967295, 2147483648, 2147483647], dtype=np.uint32)
        y = x.view(np.int32)
        expect(node, inputs=[x], outputs=[y], name="test_bitcast_uint32_to_int32")

    @staticmethod
    def export_bitcast_2d_float32_to_int32() -> None:
        """Test bitcasting 2D array from float32 to int32."""
        node = onnx.helper.make_node(
            "BitCast",
            inputs=["x"],
            outputs=["y"],
            to=onnx.TensorProto.INT32,
        )
        x = np.array([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], dtype=np.float32)
        y = x.view(np.int32)
        expect(node, inputs=[x], outputs=[y], name="test_bitcast_2d_float32_to_int32")

    @staticmethod
    def export_bitcast_int8_to_uint8() -> None:
        """Test bitcasting from int8 to uint8 (same size, different signedness)."""
        node = onnx.helper.make_node(
            "BitCast",
            inputs=["x"],
            outputs=["y"],
            to=onnx.TensorProto.UINT8,
        )
        x = np.array([-1, -128, 127, 0], dtype=np.int8)
        y = x.view(np.uint8)
        expect(node, inputs=[x], outputs=[y], name="test_bitcast_int8_to_uint8")

    @staticmethod
    def export_bitcast_scalar_float32_to_int32() -> None:
        """Test bitcasting scalar from float32 to int32."""
        node = onnx.helper.make_node(
            "BitCast",
            inputs=["x"],
            outputs=["y"],
            to=onnx.TensorProto.INT32,
        )
        x = np.array(1.0, dtype=np.float32)
        y = x.view(np.int32)
        expect(
            node, inputs=[x], outputs=[y], name="test_bitcast_scalar_float32_to_int32"
        )

    @staticmethod
    def export_bitcast_uint16_to_int16() -> None:
        """Test bitcasting from uint16 to int16 (same size, different signedness)."""
        node = onnx.helper.make_node(
            "BitCast",
            inputs=["x"],
            outputs=["y"],
            to=onnx.TensorProto.INT16,
        )
        x = np.array([1, 32768, 65535], dtype=np.uint16)
        y = x.view(np.int16)
        expect(node, inputs=[x], outputs=[y], name="test_bitcast_uint16_to_int16")

    @staticmethod
    def export_bitcast_bool_to_uint8() -> None:
        """Test bitcasting from bool to uint8 (same size)."""
        node = onnx.helper.make_node(
            "BitCast",
            inputs=["x"],
            outputs=["y"],
            to=onnx.TensorProto.UINT8,
        )
        x = np.array([True, False, True, False], dtype=np.bool_)
        y = x.view(np.uint8)
        expect(node, inputs=[x], outputs=[y], name="test_bitcast_bool_to_uint8")



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

from onnx.reference.op_run import OpRun


def construct_original_grid(data_size, align_corners):
    is_2d = len(data_size) == 2
    size_zeros = np.zeros(data_size)
    original_grid = [np.ones(data_size)]
    for dim, dim_size in enumerate(data_size):
        if align_corners == 1:
            step = 2.0 / (dim_size - 1)
            start = -1
            stop = 1 + 0.0001
            a = np.arange(start, stop, step)
        else:
            step = 2.0 / dim_size
            start = -1 + step / 2
            stop = 1
            a = np.arange(start, stop, step)
        if dim == 0:
            if is_2d:
                y = np.reshape(a, (dim_size, 1)) + size_zeros
                original_grid = [y, *original_grid]
            else:
                z = np.reshape(a, (dim_size, 1, 1)) + size_zeros
                original_grid = [z, *original_grid]
        elif dim == 1:
            if is_2d:
                x = np.reshape(a, (1, dim_size)) + size_zeros
                original_grid = [x, *original_grid]
            else:
                y = np.reshape(a, (1, dim_size, 1)) + size_zeros
                original_grid = [y, *original_grid]
        else:
            x = np.reshape(a, (1, dim_size)) + size_zeros
            original_grid = [x, *original_grid]
    return np.stack(original_grid, axis=2 if is_2d else 3)


def apply_affine_transform(theta_n, original_grid_homo):
    # theta_n: (N, 2, 3) for 2D, (N, 3, 4) for 3D
    # original_grid_homo: (H, W, 3) for 2D, (D, H, W, 4) for 3D
    assert theta_n.ndim == 3, (
        "theta_n shall have shape of (N, 2, 3) for 2D, (N, 3, 4) for 3D"
    )
    if original_grid_homo.ndim == 3:
        N, dim_2d, dim_homo = theta_n.shape
        assert dim_2d == 2 and dim_homo == 3
        H, W, dim_homo = original_grid_homo.shape
        assert dim_homo == 3
        # reshape to [H * W, dim_homo] and then transpose to [dim_homo, H * W]
        original_grid_transposed = np.transpose(
            np.reshape(original_grid_homo, (H * W, dim_homo))
        )
        grid_n = np.matmul(
            theta_n, original_grid_transposed
        )  # shape (N, dim_2d, H * W)
        # transpose to (N, H * W, dim_2d) and then reshape to (N, H, W, dim_2d)
        grid = np.reshape(np.transpose(grid_n, (0, 2, 1)), (N, H, W, dim_2d))
        return grid.astype(np.float32)
    assert original_grid_homo.ndim == 4
    N, dim_3d, dim_homo = theta_n.shape
    assert dim_3d == 3 and dim_homo == 4
    D, H, W, dim_homo = original_grid_homo.shape
    assert dim_homo == 4
    # reshape to [D * H * W, dim_homo] and then transpose to [dim_homo, D * H * W]
    original_grid_transposed = np.transpose(
        np.reshape(original_grid_homo, (D * H * W, dim_homo))
    )
    grid_n = np.matmul(
        theta_n, original_grid_transposed
    )  # shape (N, dim_3d, D * H * W)
    # transpose to (N, D * H * W, dim_3d) and then reshape to (N, D, H, W, dim_3d)
    grid = np.reshape(np.transpose(grid_n, (0, 2, 1)), (N, D, H, W, dim_3d))
    return grid.astype(np.float32)


class AffineGrid(OpRun):
    def _run(self, theta, size, align_corners=None):
        align_corners = align_corners or self.align_corners
        _, _, *data_size = size
        original_grid = construct_original_grid(data_size, align_corners)
        grid = apply_affine_transform(theta, original_grid)
        return (grid,)



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect
from onnx.helper import make_tensor


class LabelEncoder(Base):
    @staticmethod
    def export_string_int_label_encoder() -> None:
        node = onnx.helper.make_node(
            "LabelEncoder",
            inputs=["X"],
            outputs=["Y"],
            domain="ai.onnx.ml",
            keys_strings=["a", "b", "c"],
            values_int64s=[0, 1, 2],
            default_int64=42,
        )
        x = np.array(["a", "b", "d", "c", "g"]).astype(object)
        y = np.array([0, 1, 42, 2, 42]).astype(np.int64)
        expect(
            node,
            inputs=[x],
            outputs=[y],
            name="test_ai_onnx_ml_label_encoder_string_int",
        )

        node = onnx.helper.make_node(
            "LabelEncoder",
            inputs=["X"],
            outputs=["Y"],
            domain="ai.onnx.ml",
            keys_strings=["a", "b", "c"],
            values_int64s=[0, 1, 2],
        )
        x = np.array(["a", "b", "d", "c", "g"]).astype(object)
        y = np.array([0, 1, -1, 2, -1]).astype(np.int64)
        expect(
            node,
            inputs=[x],
            outputs=[y],
            name="test_ai_onnx_ml_label_encoder_string_int_no_default",
        )

    @staticmethod
    def export_tensor_based_label_encoder() -> None:
        tensor_keys = make_tensor(
            "keys_tensor", onnx.TensorProto.STRING, (3,), ["a", "b", "c"]
        )
        repeated_string_keys = ["a", "b", "c"]
        x = np.array(["a", "b", "d", "c", "g"]).astype(object)
        y = np.array([0, 1, 42, 2, 42]).astype(np.int16)

        node = onnx.helper.make_node(
            "LabelEncoder",
            inputs=["X"],
            outputs=["Y"],
            domain="ai.onnx.ml",
            keys_tensor=tensor_keys,
            values_tensor=make_tensor(
                "values_tensor", onnx.TensorProto.INT16, (3,), [0, 1, 2]
            ),
            default_tensor=make_tensor(
                "default_tensor", onnx.TensorProto.INT16, (1,), [42]
            ),
        )

        expect(
            node,
            inputs=[x],
            outputs=[y],
            name="test_ai_onnx_ml_label_encoder_tensor_mapping",
        )

        node = onnx.helper.make_node(
            "LabelEncoder",
            inputs=["X"],
            outputs=["Y"],
            domain="ai.onnx.ml",
            keys_strings=repeated_string_keys,
            values_tensor=make_tensor(
                "values_tensor", onnx.TensorProto.INT16, (3,), [0, 1, 2]
            ),
            default_tensor=make_tensor(
                "default_tensor", onnx.TensorProto.INT16, (1,), [42]
            ),
        )

        expect(
            node,
            inputs=[x],
            outputs=[y],
            name="test_ai_onnx_ml_label_encoder_tensor_value_only_mapping",
        )



from onnx.onnx_pb import (
    AttributeProto,
    SparseTensorProto,
    TensorProto,
    TensorShapeProto,
    TypeProto,
)

class InferenceError(Exception): ...

class GraphInferencer:
    def do_inferencing(
        self, input_types: list[TypeProto], input_data: list[TensorProto | None]
    ) -> list[TypeProto]: ...

class InferenceContext:
    def get_attribute(self, name: str) -> AttributeProto: ...
    def get_num_inputs(self) -> int: ...
    def get_input_type(self, idx: int) -> TypeProto: ...
    def has_input(self, idx: int) -> bool: ...
    def get_input_data(self, idx: int) -> TensorProto: ...
    def get_num_outputs(self) -> int: ...
    def get_output_type(self, idx: int) -> TypeProto: ...
    def set_output_type(self, idx: int, type_proto: TypeProto) -> bool: ...
    def has_output(self, idx: int) -> bool: ...
    def get_graph_attribute_inferencer(self, attr_name: str) -> GraphInferencer: ...
    def get_input_sparse_data(self, idx: int) -> SparseTensorProto: ...
    def get_symbolic_input(self, idx: int) -> TensorShapeProto: ...
    def get_display_name(self) -> str: ...

def infer_shapes(
    b: bytes, check_type: bool, strict_mode: bool, data_prop: bool
) -> bytes: ...
def infer_shapes_path(
    model_path: str,
    output_path: str,
    check_type: bool,
    strict_mode: bool,
    data_prop: bool,
) -> None: ...
def infer_function_output_types(
    b: bytes,
    input_types: list[bytes],
    attributes: list[bytes],
) -> list[bytes]: ...



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

from onnx.reference.op_run import OpRun


class CumProd(OpRun):
    def _run(self, x, axis, exclusive=None, reverse=None):
        axis = np.asarray(axis)
        if axis.ndim != 0:
            raise ValueError(f"Axis must be a rank-0 tensor, got `{axis.ndim}`.")
        if reverse:
            rev_indices = [slice(0, s) for s in x.shape]
            rev_indices[axis] = slice(None, None, -1)
            x = x[tuple(rev_indices)]
        if exclusive:
            indices_c = [slice(0, s) for s in x.shape]
            indices_d = [slice(0, s) for s in x.shape]
            indices_c[axis] = slice(0, -1)
            indices_d[axis] = slice(1, x.shape[axis])
            res = np.ones(x.shape, dtype=x.dtype)
            np.cumprod(x[tuple(indices_c)], axis=axis, out=res[tuple(indices_d)])
        else:
            res = np.cumprod(x, axis=axis, dtype=x.dtype)
        if reverse:
            res = res[tuple(rev_indices)]
        return (res,)



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0


from __future__ import annotations

import numpy as np

import onnx
from onnx import TensorProto
from onnx.helper import (
    np_dtype_to_tensor_dtype,
    tensor_dtype_to_np_dtype,
)
from onnx.reference.op_run import OpRun

_QUANT_TYPES = {
    TensorProto.UINT8,
    TensorProto.INT8,
    TensorProto.UINT16,
    TensorProto.INT16,
    TensorProto.INT2,
    TensorProto.UINT2,
    TensorProto.UINT4,
    TensorProto.INT4,
    TensorProto.FLOAT8E4M3FN,
    TensorProto.FLOAT8E4M3FNUZ,
    TensorProto.FLOAT8E5M2,
    TensorProto.FLOAT8E5M2FNUZ,
    TensorProto.FLOAT4E2M1,
}

_QUANT_INTEGER_RANGES = {
    TensorProto.UINT8: (0, 255),
    TensorProto.INT8: (-128, 127),
    TensorProto.UINT16: (0, 65535),
    TensorProto.INT16: (-32768, 32767),
    TensorProto.UINT4: (0, 15),
    TensorProto.INT4: (-8, 7),
    TensorProto.UINT2: (0, 3),
    TensorProto.INT2: (-2, 1),
}


def _reshape_input(
    value: np.ndarray,
    shape: tuple[int, ...],
    axis: int,
    block_size: int | None = None,
) -> np.ndarray:
    """Reshape/Replicate scale/zero-point to be broadcastable to shape.

    Args:
        value: the array to be reshaped/replicated
        shape: the target shape
        axis: quantization axis, applicable for per-axis and blocked quantization
        block_size: size of quantization block, applicable only for blocked quantization

    Returns:
        value array after reshape/replicate according to quantization mode.
    """
    if len(value.shape) == 0:
        return value
    if len(value.shape) > 0 and value.size == 1:
        return value[0]
    if not block_size:
        assert len(value.shape) == 1
        dims = [1] * len(shape)
        try:
            dims[axis] = value.size
            return value.reshape(tuple(dims))
        except IndexError as e:
            raise IndexError(
                f"axis is out of boundary, axis={axis}, "
                f"value.shape={value.shape}, shape={shape}."
            ) from e

    if block_size <= 0:
        raise ValueError("block_size must be a positive integer.")

    # repeat scale to get element-wise scale
    value = np.repeat(value, repeats=block_size, axis=axis)
    if (
        shape[axis] != value.shape[axis]
    ):  # block_size does not divide x, handle the remainder block
        value = value.take(indices=range(shape[axis]), axis=axis)
    if value.shape != shape:
        raise ValueError(
            "Invalid shapes for Blocked Quantization. Input 2 shape should identical to Input 1 shape, except for one dimension, in which blocking is performed"
        )
    assert np.broadcast_shapes(shape, value.shape) == shape
    return value


class _CommonQuantizeLinear(OpRun):
    def _run(
        self,
        x: np.ndarray,
        y_scale: np.ndarray,
        zero_point: np.ndarray | None = None,
        axis: int = 1,
        saturate: bool = True,
        block_size: int | None = None,
        output_dtype: TensorProto.DataType | None = None,
        precision: int | None = None,
    ) -> tuple[np.ndarray]:
        y_scale = _reshape_input(y_scale, x.shape, axis, block_size)

        # Determine output data type
        tensor_type = output_dtype
        if zero_point is not None:
            zero_point_type = np_dtype_to_tensor_dtype(zero_point.dtype)
            if output_dtype and output_dtype != zero_point_type:
                raise ValueError(
                    f"Mismatched output data-types: output_dtype={output_dtype}, zero_point type={zero_point_type}"
                )
            tensor_type = zero_point_type
        tensor_type = tensor_type or TensorProto.UINT8

        if tensor_type not in _QUANT_TYPES:
            raise ValueError(
                f"Unexpected type: output_dtype={tensor_type} is not a supported quantized type."
            )

        # Compute
        zero_point = (
            _reshape_input(zero_point, x.shape, axis, block_size)
            if zero_point is not None
            else 0
        )
        if precision:
            precision_np = tensor_dtype_to_np_dtype(precision)
            x = x.astype(precision_np) / y_scale.astype(precision_np)
        else:
            x = x / y_scale

        if tensor_type in _QUANT_INTEGER_RANGES:
            xi = np.rint(x).astype(np.int32)
            xi += zero_point
            dtype = tensor_dtype_to_np_dtype(tensor_type)
            quant_range = _QUANT_INTEGER_RANGES[tensor_type]
            return (np.clip(xi, quant_range[0], quant_range[1]).astype(dtype),)

        if tensor_type in {
            TensorProto.FLOAT8E4M3FN,
            TensorProto.FLOAT8E4M3FNUZ,
            TensorProto.FLOAT8E5M2,
            TensorProto.FLOAT8E5M2FNUZ,
        }:
            if saturate:
                return (
                    onnx.numpy_helper.saturate_cast(
                        x, dtype=tensor_dtype_to_np_dtype(tensor_type)
                    ),
                )
            return (x.astype(tensor_dtype_to_np_dtype(tensor_type)),)

        if tensor_type == TensorProto.FLOAT4E2M1:
            x += zero_point
            return (x.astype(tensor_dtype_to_np_dtype(tensor_type)),)

        raise ValueError(
            f"Unexpected type: output_dtype={tensor_type} is not a supported quantized type."
        )


class QuantizeLinear_10(_CommonQuantizeLinear):
    def _run(self, x, y_scale, zero_point=None, axis: int = 1):
        if len(y_scale.shape) > 1:
            raise ValueError("Input 2 must be a vector or a number.")
        return super()._run(x, y_scale, zero_point, axis=axis)


class QuantizeLinear_19(_CommonQuantizeLinear):
    def _run(self, x, y_scale, zero_point=None, axis: int = 1, saturate: bool = True):
        if len(y_scale.shape) > 1:
            raise ValueError("Input 2 must be a vector or a number.")
        return super()._run(x, y_scale, zero_point, axis=axis, saturate=saturate)


class QuantizeLinear_21(_CommonQuantizeLinear):
    def _run(
        self,
        *args,
        axis: int = 1,
        saturate: bool = True,
        block_size: int = 0,
        output_dtype=None,
    ):
        # args: x, y_scale, zero_point
        return super()._run(
            *args,
            axis=axis,
            saturate=saturate,
            block_size=block_size,
            output_dtype=output_dtype,
        )


class QuantizeLinear_23(_CommonQuantizeLinear):
    def _run(
        self,
        *args,
        axis: int = 1,
        saturate: bool = True,
        block_size: int = 0,
        output_dtype=None,
        precision=None,
    ):
        # args: x, y_scale, zero_point
        return super()._run(
            *args,
            axis=axis,
            saturate=saturate,
            block_size=block_size,
            output_dtype=output_dtype,
            precision=precision,
        )


class QuantizeLinear_25(_CommonQuantizeLinear):
    def _run(
        self,
        *args,
        axis: int = 1,
        saturate: bool = True,
        block_size: int = 0,
        output_dtype=None,
        precision=None,
    ):
        # args: x, y_scale, zero_point
        return super()._run(
            *args,
            axis=axis,
            saturate=saturate,
            block_size=block_size,
            output_dtype=output_dtype,
            precision=precision,
        )



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np
from numpy.random import RandomState

from onnx.reference.op_run import OpRun


def _dropout(
    X: np.ndarray,
    drop_probability: float = 0.5,
    seed: int | None = None,
    training_mode: bool = False,
    return_mask: bool = False,
) -> tuple[np.ndarray]:
    if drop_probability == 0 or not training_mode:
        if return_mask:
            return X, np.ones(X.shape, dtype=bool)
        return (X,)

    rnd = RandomState(seed)
    mask = rnd.uniform(0, 1.0, X.shape) >= drop_probability
    scale = 1.0 / (1.0 - drop_probability)
    return (mask * X * scale, mask.astype(bool)) if return_mask else (mask * X * scale,)


class DropoutBase(OpRun):
    def __init__(self, onnx_node, run_params):
        OpRun.__init__(self, onnx_node, run_params)
        self.n_outputs = len(onnx_node.output)

    def _private_run(
        self,
        X: np.ndarray,
        seed: int | None = None,
        ratio: float = 0.5,
        training_mode: bool = False,
    ) -> tuple[np.ndarray]:
        return _dropout(
            X,
            ratio,
            seed=seed,
            return_mask=self.n_outputs == 2,
            training_mode=training_mode,
        )


class Dropout_7(DropoutBase):
    def _run(self, X, ratio=None):
        return self._private_run(X, ratio)


class Dropout_12(DropoutBase):
    def _run(self, *inputs, seed=None):
        X = inputs[0]
        ratio = 0.5 if len(inputs) <= 1 else inputs[1]
        training_mode = False if len(inputs) <= 2 else inputs[2]
        return self._private_run(
            X,
            seed=seed,
            ratio=ratio,
            training_mode=training_mode,
        )



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

from onnx.reference.op_run import OpRun


class Shape_1(OpRun):
    def _run(self, data):
        return (np.array(data.shape, dtype=np.int64),)


class Shape_15(Shape_1):
    @staticmethod
    def _interval(n: int, start: int | None, end: int | None) -> tuple[int, int] | None:
        if start == 0:
            if end is None or np.isnan(end):
                return None
            if end < 0:
                return (0, n + end)
            return (0, end)
        if end is None or np.isnan(end):
            return (start, n)
        if end < 0:
            return (start, n + end)
        return (start, end)

    def _run(self, data, end=None, start=None):
        ab = self._interval(len(data.shape), start=start, end=end)
        if ab is None:
            return (np.array(data.shape, dtype=np.int64),)
        return (np.array(data.shape[ab[0] : ab[1]], dtype=np.int64),)



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect


class BitShift(Base):
    @staticmethod
    def export_right_unit8() -> None:
        node = onnx.helper.make_node(
            "BitShift", inputs=["x", "y"], outputs=["z"], direction="RIGHT"
        )

        x = np.array([16, 4, 1]).astype(np.uint8)
        y = np.array([1, 2, 3]).astype(np.uint8)
        z = x >> y  # expected output [8, 1, 0]
        expect(node, inputs=[x, y], outputs=[z], name="test_bitshift_right_uint8")

    @staticmethod
    def export_right_unit16() -> None:
        node = onnx.helper.make_node(
            "BitShift", inputs=["x", "y"], outputs=["z"], direction="RIGHT"
        )

        x = np.array([16, 4, 1]).astype(np.uint16)
        y = np.array([1, 2, 3]).astype(np.uint16)
        z = x >> y  # expected output [8, 1, 0]
        expect(node, inputs=[x, y], outputs=[z], name="test_bitshift_right_uint16")

    @staticmethod
    def export_right_unit32() -> None:
        node = onnx.helper.make_node(
            "BitShift", inputs=["x", "y"], outputs=["z"], direction="RIGHT"
        )

        x = np.array([16, 4, 1]).astype(np.uint32)
        y = np.array([1, 2, 3]).astype(np.uint32)
        z = x >> y  # expected output [8, 1, 0]
        expect(node, inputs=[x, y], outputs=[z], name="test_bitshift_right_uint32")

    @staticmethod
    def export_right_unit64() -> None:
        node = onnx.helper.make_node(
            "BitShift", inputs=["x", "y"], outputs=["z"], direction="RIGHT"
        )

        x = np.array([16, 4, 1]).astype(np.uint64)
        y = np.array([1, 2, 3]).astype(np.uint64)
        z = x >> y  # expected output [8, 1, 0]
        expect(node, inputs=[x, y], outputs=[z], name="test_bitshift_right_uint64")

    @staticmethod
    def export_left_unit8() -> None:
        node = onnx.helper.make_node(
            "BitShift", inputs=["x", "y"], outputs=["z"], direction="LEFT"
        )

        x = np.array([16, 4, 1]).astype(np.uint8)
        y = np.array([1, 2, 3]).astype(np.uint8)
        z = x << y  # expected output [32, 16, 8]
        expect(node, inputs=[x, y], outputs=[z], name="test_bitshift_left_uint8")

    @staticmethod
    def export_left_unit16() -> None:
        node = onnx.helper.make_node(
            "BitShift", inputs=["x", "y"], outputs=["z"], direction="LEFT"
        )

        x = np.array([16, 4, 1]).astype(np.uint16)
        y = np.array([1, 2, 3]).astype(np.uint16)
        z = x << y  # expected output [32, 16, 8]
        expect(node, inputs=[x, y], outputs=[z], name="test_bitshift_left_uint16")

    @staticmethod
    def export_left_unit32() -> None:
        node = onnx.helper.make_node(
            "BitShift", inputs=["x", "y"], outputs=["z"], direction="LEFT"
        )

        x = np.array([16, 4, 1]).astype(np.uint32)
        y = np.array([1, 2, 3]).astype(np.uint32)
        z = x << y  # expected output [32, 16, 8]
        expect(node, inputs=[x, y], outputs=[z], name="test_bitshift_left_uint32")

    @staticmethod
    def export_left_unit64() -> None:
        node = onnx.helper.make_node(
            "BitShift", inputs=["x", "y"], outputs=["z"], direction="LEFT"
        )

        x = np.array([16, 4, 1]).astype(np.uint64)
        y = np.array([1, 2, 3]).astype(np.uint64)
        z = x << y  # expected output [32, 16, 8]
        expect(node, inputs=[x, y], outputs=[z], name="test_bitshift_left_uint64")



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import os
import pathlib
import platform
import sys
import sysconfig
import tempfile
import unittest
from unittest.mock import patch


# Extract get_python_execute function from setup.py for testing
def get_python_execute() -> str:
    """Get the Python executable path for CMake configuration.

    Prefer sys.executable as it represents the currently running Python.
    Only fall back to directory traversal if sys.executable is invalid.
    """
    WINDOWS = os.name == "nt"

    if WINDOWS:
        return sys.executable

    # First, check if sys.executable is valid and usable
    if os.path.isfile(sys.executable) and os.access(sys.executable, os.X_OK):
        return sys.executable

    # Fallback: Try to search for Python based on include path
    # This addresses https://github.com/python/cpython/issues/84399
    python_dir = os.path.abspath(
        os.path.join(sysconfig.get_path("include"), "..", "..")
    )
    if os.path.isdir(python_dir):
        python_bin = os.path.join(python_dir, "bin", "python3")
        if os.path.isfile(python_bin):
            return python_bin
        python_bin = os.path.join(python_dir, "bin", "python")
        if os.path.isfile(python_bin):
            return python_bin

    return sys.executable


class TestGetPythonExecutable(unittest.TestCase):
    """Test suite for get_python_execute() function from setup.py."""

    def test_windows_returns_sys_executable(self) -> None:
        """On Windows, get_python_execute() should always return sys.executable."""
        with patch("os.name", "nt"):
            result = get_python_execute()
            self.assertEqual(result, sys.executable)

    def test_valid_sys_executable_is_preferred(self) -> None:
        """When sys.executable is valid, it should be returned (non-Windows)."""
        with patch("os.name", "posix"):
            # sys.executable should be valid in most test environments
            result = get_python_execute()
            # Should return sys.executable since it's valid
            self.assertEqual(result, sys.executable)

    @unittest.skipIf(
        sys.platform == "win32" and sys.version_info < (3, 11),
        "On Windows this test requires Python >= 3.11 due to sysconfig/sys.abiflags behavior on older interpreters",
    )
    def test_invalid_sys_executable_falls_back(self) -> None:
        """When sys.executable is invalid, should fall back to directory search."""
        with (
            patch("os.name", "posix"),
            patch("sys.executable", "/nonexistent/python"),
            patch("os.path.isfile") as mock_isfile,
            patch("os.access") as mock_access,
            patch("os.path.isdir") as mock_isdir,
        ):
            # Mock sys.executable as invalid
            mock_isfile.return_value = False
            mock_access.return_value = False
            mock_isdir.return_value = False

            result = get_python_execute()
            # Should fall back to sys.executable as last resort
            self.assertEqual(result, "/nonexistent/python")

    def test_fallback_finds_python3_in_bin(self) -> None:
        """Test fallback finds python3 in bin directory when sys.executable is invalid."""
        # Create a temporary directory structure for testing
        with tempfile.TemporaryDirectory() as tmpdir:
            mock_python_dir = pathlib.Path(tmpdir) / "python_install"
            mock_bin_dir = mock_python_dir / "bin"
            # Include path typically goes: <prefix>/include/pythonX.Y
            # Going up two dirs (.. ..) gets us back to prefix
            mock_include_dir = mock_python_dir / "include" / "python3.12"
            mock_bin_dir.mkdir(parents=True)
            mock_include_dir.mkdir(parents=True)
            mock_python3 = mock_bin_dir / "python3"
            # Create actual executable file
            mock_python3.touch(mode=0o755)

            with (
                patch("os.name", "posix"),
                patch("sys.executable", "/invalid/python"),
                patch("sysconfig.get_path") as mock_get_path,
            ):
                # Setup mocks - return the include/python3.12 path
                mock_get_path.return_value = str(mock_include_dir)

                result = get_python_execute()
                self.assertEqual(result, str(mock_python3))

    def test_fallback_finds_python_in_bin(self) -> None:
        """Test fallback finds python (not python3) when python3 doesn't exist."""
        # Create a temporary directory structure for testing
        with tempfile.TemporaryDirectory() as tmpdir:
            mock_python_dir = pathlib.Path(tmpdir) / "python_install"
            mock_bin_dir = mock_python_dir / "bin"
            # Include path typically goes: <prefix>/include/pythonX.Y
            # Going up two dirs (.. ..) gets us back to prefix
            mock_include_dir = mock_python_dir / "include" / "python3.12"
            mock_bin_dir.mkdir(parents=True)
            mock_include_dir.mkdir(parents=True)
            mock_python = mock_bin_dir / "python"
            # Create actual executable file (python3 doesn't exist)
            mock_python.touch(mode=0o755)

            with (
                patch("os.name", "posix"),
                patch("sys.executable", "/invalid/python"),
                patch("sysconfig.get_path") as mock_get_path,
            ):
                # Setup mocks - return the include/python3.12 path
                mock_get_path.return_value = str(mock_include_dir)

                result = get_python_execute()
                self.assertEqual(result, str(mock_python))

    @unittest.skipIf(
        sys.platform == "win32" and sys.version_info < (3, 11),
        "On Windows this test requires Python >= 3.11 due to sysconfig/sys.abiflags behavior on older interpreters",
    )
    def test_executable_permission_check(self) -> None:
        """Test that executable permission is verified for sys.executable."""
        with (
            patch("os.name", "posix"),
            patch("sys.executable", "/path/to/python"),
            patch("os.path.isfile") as mock_isfile,
            patch("os.access") as mock_access,
            patch("os.path.isdir") as mock_isdir,
        ):
            # File exists but is not executable
            mock_isfile.return_value = True
            mock_access.return_value = False
            mock_isdir.return_value = False

            result = get_python_execute()
            # Should fall back to sys.executable even though it's not executable
            self.assertEqual(result, sys.executable)

    def test_real_environment(self) -> None:
        """Test with real environment to ensure it works in practice."""
        result = get_python_execute()

        # Result should be a non-empty string
        self.assertIsInstance(result, str)
        self.assertGreater(len(result), 0)

        # On Windows, should be exactly sys.executable
        if platform.system() == "Windows":
            self.assertEqual(result, sys.executable)
        # On non-Windows, in a normal environment, should return sys.executable
        # since it should be valid
        elif os.path.isfile(sys.executable) and os.access(sys.executable, os.X_OK):
            self.assertEqual(result, sys.executable)

    def test_virtual_environment_detection(self) -> None:
        """Test that virtual environment Python is correctly detected."""
        with patch("os.name", "posix"):
            result = get_python_execute()

            # In a venv or virtualenv, sys.executable should point to the venv Python
            # and the function should return it
            if hasattr(sys, "prefix") and hasattr(sys, "base_prefix"):
                in_venv = sys.prefix != sys.base_prefix
                if in_venv and os.path.isfile(sys.executable):
                    self.assertEqual(
                        result,
                        sys.executable,
                        "Virtual environment Python should be detected",
                    )

    @unittest.skipIf(
        platform.system() == "Windows",
        "Fallback mechanism only applies to POSIX systems",
    )
    def test_cpython_issue_84399_fallback(self) -> None:
        """Test that the fallback handles cpython issue #84399 edge case.

        This test is skipped on Windows because the Windows implementation
        always returns sys.executable without any fallback logic.
        The fallback mechanism is only relevant for POSIX systems.
        """
        # This test verifies that the fallback mechanism is still present
        # for the edge case mentioned in https://github.com/python/cpython/issues/84399
        with (
            patch("sys.executable", "/usr/bin/python-invalid"),
            patch("os.path.isfile") as mock_isfile,
            patch("os.access") as mock_access,
            patch("sysconfig.get_path") as mock_get_path,
            patch("os.path.isdir") as mock_isdir,
            patch("os.path.abspath") as mock_abspath,
            patch("os.path.join", side_effect=lambda *args: "/".join(args)),
        ):
            # sys.executable points to invalid path
            mock_isfile.return_value = False
            mock_access.return_value = False
            mock_get_path.return_value = "/usr/include/python3.12"
            mock_isdir.return_value = True
            mock_abspath.return_value = "/usr"

            # Mock finding python3 in /usr/bin
            def isfile_check(path: str) -> bool:
                return path == "/usr/bin/python3"

            mock_isfile.side_effect = isfile_check

            result = get_python_execute()
            self.assertEqual(result, "/usr/bin/python3")


if __name__ == "__main__":
    unittest.main()



# SPDX-License-Identifier: Apache-2.0

# Copyright (c) ONNX Project Contributors
from __future__ import annotations

import unittest

import parameterized

import onnx.helper
import onnx.shape_inference


class NodeInferenceTest(unittest.TestCase):
    @parameterized.parameterized.expand(
        [
            ("GreaterOrEqual",),
            ("LessOrEqual",),
        ]
    )
    def test_comparison_op(self, op_type):
        node = onnx.helper.make_node(op_type, ["x", "y"], ["z"])
        schema = onnx.defs.get_schema(node.op_type, 23, "")
        xtype = onnx.helper.make_tensor_type_proto(onnx.TensorProto.INT32, [1, 10])
        ytype = onnx.helper.make_tensor_type_proto(onnx.TensorProto.INT32, [10, 1])
        result = onnx.shape_inference.infer_node_outputs(
            schema, node, {"x": xtype, "y": ytype}
        )
        self.assertEqual(list(result.keys()), ["z"])
        self.assertEqual(result["z"].tensor_type.elem_type, onnx.TensorProto.BOOL)
        self.assertEqual(
            [dim.dim_value for dim in result["z"].tensor_type.shape.dim],
            [10, 10],
        )


if __name__ == "__main__":
    unittest.main()



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect


def triu_reference_implementation(x, k=0):
    return np.triu(x, k)


def tril_reference_implementation(x, k=0):
    return np.tril(x, k)


class Trilu(Base):
    @staticmethod
    def export_triu() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x"],
            outputs=["y"],
        )

        x = np.random.randint(10, size=(4, 5)).astype(np.int64)
        # X:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 0, 8, 7],
        #   [4, 3, 4, 2, 4]]
        # expect result:
        #  [[4, 7, 3, 7, 9],
        #   [0, 2, 8, 6, 9],
        #   [0, 0, 0, 8, 7],
        #   [0, 0, 0, 2, 4]]
        y = triu_reference_implementation(x)
        expect(node, inputs=[x], outputs=[y], name="test_triu")

    @staticmethod
    def export_triu_neg() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
        )

        x = np.random.randint(10, size=(4, 5)).astype(np.int64)
        k = np.array(-1).astype(np.int64)
        # X:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 0, 8, 7],
        #   [4, 3, 4, 2, 4]]
        # expect result:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [0, 4, 0, 8, 7],
        #   [0, 0, 4, 2, 4]]
        y = triu_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_neg")

    @staticmethod
    def export_triu_out_neg_out() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
        )

        x = np.random.randint(10, size=(4, 5)).astype(np.int64)
        k = np.array(-7).astype(np.int64)
        # X:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 0, 8, 7],
        #   [4, 3, 4, 2, 4]]
        # expect result:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 0, 8, 7],
        #   [4, 3, 4, 2, 4]]
        y = triu_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_out_neg_out")

    @staticmethod
    def export_triu_pos() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
        )

        x = np.random.randint(10, size=(4, 5)).astype(np.int64)
        k = np.array(2).astype(np.int64)
        # X:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 0, 8, 7],
        #   [4, 3, 4, 2, 4]]
        # expect result:
        #  [[0, 0, 3, 7, 9],
        #   [0, 0, 0, 6, 9],
        #   [0, 0, 0, 0, 7],
        #   [0, 0, 0, 0, 0]]
        y = triu_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_pos")

    @staticmethod
    def export_triu_out_pos() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
        )

        x = np.random.randint(10, size=(4, 5)).astype(np.int64)
        k = np.array(6).astype(np.int64)
        # X:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 0, 8, 7],
        #   [4, 3, 4, 2, 4]]
        # expect result:
        #  [[0, 0, 0, 0, 0],
        #   [0, 0, 0, 0, 0],
        #   [0, 0, 0, 0, 0],
        #   [0, 0, 0, 0, 0]]
        y = triu_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_out_pos")

    @staticmethod
    def export_triu_square() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x"],
            outputs=["y"],
        )

        x = np.random.randint(10, size=(2, 3, 3)).astype(np.int64)
        y = triu_reference_implementation(x)
        # X:
        # [[[4, 6, 9],
        #   [7, 5, 4],
        #   [8, 1, 2]],
        #
        #  [[1, 4, 9],
        #   [9, 6, 3],
        #   [8, 9, 8]]]
        # expect result:
        # [[[4, 6, 9],
        #   [0, 5, 4],
        #   [0, 0, 2]],
        #
        #  [[1, 4, 9],
        #   [0, 6, 3],
        #   [0, 0, 8]]]
        expect(node, inputs=[x], outputs=[y], name="test_triu_square")

    @staticmethod
    def export_triu_square_neg() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
        )

        x = np.random.randint(10, size=(2, 3, 3)).astype(np.int64)
        k = np.array(-1).astype(np.int64)
        # X:
        # [[[4, 6, 9],
        #   [7, 5, 4],
        #   [8, 1, 2]],
        #
        #  [[1, 4, 9],
        #   [9, 6, 3],
        #   [8, 9, 8]]]
        # expect result:
        # [[[4, 6, 9],
        #   [7, 5, 4],
        #   [0, 1, 2]],
        #
        #  [[1, 4, 9],
        #   [9, 6, 3],
        #   [0, 9, 8]]]
        y = triu_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_square_neg")

    @staticmethod
    def export_triu_one_row() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
        )

        x = np.random.randint(10, size=(3, 1, 5)).astype(np.int64)
        k = np.array(1).astype(np.int64)
        # X:
        # [[[1, 4, 9, 7, 1]],
        #
        #  [[9, 2, 8, 8, 4]],
        #
        #  [[3, 9, 7, 4, 2]]]
        # expect result:
        # [[[0, 4, 9, 7, 1]],
        #
        #  [[0, 2, 8, 8, 4]],
        #
        #  [[0, 9, 7, 4, 2]]]
        y = triu_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_one_row")

    @staticmethod
    def export_triu_zero() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
        )

        x = np.random.randint(10, size=(0, 5)).astype(np.int64)
        k = np.array(6).astype(np.int64)
        # X:
        # []
        # expect result:
        # []
        y = triu_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_triu_zero")

    @staticmethod
    def export_tril() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x"],
            outputs=["y"],
            upper=0,
        )

        x = np.random.randint(10, size=(4, 5)).astype(np.int64)
        # X:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 1, 8, 7],
        #   [4, 3, 4, 2, 4]]
        # expect result:
        #  [[4, 0, 0, 0, 0],
        #   [1, 2, 0, 0, 0],
        #   [9, 4, 1, 0, 0],
        #   [4, 3, 4, 2, 0]]
        y = tril_reference_implementation(x)
        expect(node, inputs=[x], outputs=[y], name="test_tril")

    @staticmethod
    def export_tril_neg() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
        )

        x = np.random.randint(10, size=(4, 5)).astype(np.int64)
        k = np.array(-1).astype(np.int64)
        # X:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 1, 8, 7],
        #   [4, 3, 4, 2, 4]]
        # expect result:
        #  [[0, 0, 0, 0, 0],
        #   [1, 0, 0, 0, 0],
        #   [9, 4, 0, 0, 0],
        #   [4, 3, 4, 0, 0]]
        y = tril_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_neg")

    @staticmethod
    def export_tril_out_neg() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
        )

        x = np.random.randint(10, size=(4, 5)).astype(np.int64)
        k = np.array(-7).astype(np.int64)
        # X:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 1, 8, 7],
        #   [4, 3, 4, 2, 4]]
        # expect result:
        #  [[0, 0, 0, 0, 0],
        #   [0, 0, 0, 0, 0],
        #   [0, 0, 0, 0, 0],
        #   [0, 0, 0, 0, 0]]
        y = tril_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_out_neg")

    @staticmethod
    def export_tril_pos() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
        )

        x = np.random.randint(10, size=(4, 5)).astype(np.int64)
        k = np.array(2).astype(np.int64)
        # X:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 1, 8, 7],
        #   [4, 3, 4, 2, 4]]
        # expect result:
        #  [[4, 7, 3, 0, 0],
        #   [1, 2, 8, 6, 0],
        #   [9, 4, 1, 8, 7],
        #   [4, 3, 4, 2, 4]]
        y = tril_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_pos")

    @staticmethod
    def export_tril_out_pos() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
        )
        x = np.random.randint(10, size=(4, 5)).astype(np.int64)
        k = np.array(6).astype(np.int64)
        # X:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 1, 8, 7],
        #   [4, 3, 4, 2, 4]]
        # expect result:
        #  [[4, 7, 3, 7, 9],
        #   [1, 2, 8, 6, 9],
        #   [9, 4, 1, 8, 7],
        #   [4, 3, 4, 2, 4]]
        y = tril_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_out_pos")

    @staticmethod
    def export_tril_square() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x"],
            outputs=["y"],
            upper=0,
        )

        x = np.random.randint(10, size=(2, 3, 3)).astype(np.int64)
        # X:
        # [[[0, 4, 3],
        #   [2, 0, 9],
        #   [8, 2, 5]],
        #
        #  [[2, 7, 2],
        #   [2, 6, 0],
        #   [2, 6, 5]]]
        # expect result:
        # [[[0, 0, 0],
        #   [2, 0, 0],
        #   [8, 2, 5]],
        #
        #  [[2, 0, 0],
        #   [2, 6, 0],
        #   [2, 6, 5]]]
        y = tril_reference_implementation(x)
        expect(node, inputs=[x], outputs=[y], name="test_tril_square")

    @staticmethod
    def export_tril_square_neg() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
        )

        x = np.random.randint(10, size=(2, 3, 3)).astype(np.int64)
        k = np.array(-1).astype(np.int64)
        # X:
        # [[[0, 4, 3],
        #   [2, 0, 9],
        #   [8, 2, 5]],
        #
        #  [[2, 7, 2],
        #   [2, 6, 0],
        #   [2, 6, 5]]]
        # expect result:
        # [[[0, 0, 0],
        #   [2, 0, 0],
        #   [8, 2, 0]],
        #
        #  [[0, 0, 0],
        #   [2, 0, 0],
        #   [2, 6, 0]]]
        y = tril_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_square_neg")

    @staticmethod
    def export_tril_one_row() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x"],
            outputs=["y"],
            upper=0,
        )

        x = np.random.randint(10, size=(3, 1, 5)).astype(np.int64)
        # X:
        # [[[6, 2, 4, 1, 6]],
        #
        #  [[8, 3, 8, 7, 0]],
        #
        #  [[2, 2, 9, 5, 9]]]
        # expect result:
        # [[[6, 0, 0, 0, 0]],
        #
        #  [[8, 0, 0, 0, 0]],
        #
        #  [[2, 0, 0, 0, 0]]]
        y = tril_reference_implementation(x)
        expect(node, inputs=[x], outputs=[y], name="test_tril_one_row_neg")

    @staticmethod
    def export_tril_zero() -> None:
        node = onnx.helper.make_node(
            "Trilu",
            inputs=["x", "k"],
            outputs=["y"],
            upper=0,
        )

        x = np.random.randint(10, size=(3, 0, 5)).astype(np.int64)
        k = np.array(6).astype(np.int64)
        # X:
        # []
        # expect result:
        # []
        y = tril_reference_implementation(x, int(k))
        expect(node, inputs=[x, k], outputs=[y], name="test_tril_zero")
